package cn.com.ctop.module.job.distribute.handler;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.log.XxlJobLogger;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;

@Component
public class KuaishouHourlyJob {

    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;
    /**
     * 快手账户时报
     */
    @XxlJob("kuaishouAccountHourlyJob")
    public ReturnT<String> kuaishouAccountHourlyJob(String params) throws Exception {
        if(null == params||params.trim().equals("")){
            XxlJobLogger.log("获取快手账户时报数据异常：参数不存在");
            return ReturnT.SUCCESS;
        }
        Long accountId = Long.parseLong(params);
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        if(null == token){
            XxlJobLogger.log("获取快手账户时报数据异常：token数据不存在");
            return ReturnT.SUCCESS;
        }
        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);
        if ("00".equals(hour)) {
            getDate = DateUtils.addDay(getDate, -1);
        }
        //1:查询当日数据
        Date finalGetDate = getDate;
        try {
            kuaishouInterfaceService.getAdvertiserReportHourly(token, finalGetDate, finalGetDate);
            XxlJobLogger.log("获取快手账户时报数据成功");
            return ReturnT.SUCCESS;
        }catch (Exception e){
            XxlJobLogger.log("获取快手账户时报数据失败:服务器内部错误");
            return ReturnT.FAIL;
        }
    }

    /**
     * 快手计划时报
     */
    @XxlJob("kuaishouPlanHourlyJob")
    public ReturnT<String> kuaishouPlanHourlyJob(String params) throws Exception {
        if(null == params||params.trim().equals("")){
            XxlJobLogger.log("获取快手计划时报数据异常：参数不存在");
            return ReturnT.SUCCESS;
        }
        Long accountId = Long.parseLong(params);
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        if(null == token){
            XxlJobLogger.log("获取快手计划时报数据异常：token数据不存在");
            return ReturnT.SUCCESS;
        }
        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);
        if ("00".equals(hour)) {
            getDate = DateUtils.addDay(getDate, -1);
        }
        //1:查询当日数据
        Date finalGetDate = getDate;
        try {
            kuaishouInterfaceService.getAdvertiserCampaignReportHourly(token, finalGetDate, finalGetDate);
            XxlJobLogger.log("获取快手计划时报数据成功");
            return ReturnT.SUCCESS;
        }catch (Exception e){
            XxlJobLogger.log("获取快手计划时报数据失败:服务器内部错误");
            return ReturnT.FAIL;
        }
    }

    /**
     * 快手组时报
     */
    @XxlJob("kuaishouGroupHourlyJob")
    public ReturnT<String> kuaishouGroupHourlyJob(String params) throws Exception {
        if(null == params||params.trim().equals("")){
            XxlJobLogger.log("获取快手组时报数据异常：参数不存在");
            return ReturnT.SUCCESS;
        }
        Long accountId = Long.parseLong(params);
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        if(null == token){
            XxlJobLogger.log("获取快手组时报数据异常：token数据不存在");
            return ReturnT.SUCCESS;
        }
        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);
        if ("00".equals(hour)) {
            getDate = DateUtils.addDay(getDate, -1);
        }
        //1:查询当日数据
        Date finalGetDate = getDate;
        try {
            kuaishouInterfaceService.getAdvertiserGroupReportHourly(token, finalGetDate, finalGetDate);
            XxlJobLogger.log("获取快手组时报数据成功");
            return ReturnT.SUCCESS;
        }catch (Exception e){
            XxlJobLogger.log("获取快手组时报数据失败:服务器内部错误");
            return ReturnT.FAIL;
        }
    }

    /**
     * 快手组时报
     */
    @XxlJob("kuaishouCreativeHourlyJob")
    public ReturnT<String> kuaishouCreativeHourlyJob(String params) throws Exception {
        if(null == params||params.trim().equals("")){
            XxlJobLogger.log("获取快手创意时报数据异常：参数不存在");
            return ReturnT.SUCCESS;
        }
        Long accountId = Long.parseLong(params);
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        if(null == token){
            XxlJobLogger.log("获取快手创意时报数据异常：token数据不存在");
            return ReturnT.SUCCESS;
        }
        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);
        if ("00".equals(hour)) {
            getDate = DateUtils.addDay(getDate, -1);
        }
        //1:查询当日数据
        Date finalGetDate = getDate;
        try {
            kuaishouInterfaceService.getAdvertiserCreativeReportHourly(token, finalGetDate, finalGetDate);
            XxlJobLogger.log("获取快手创意时报数据成功");
            return ReturnT.SUCCESS;
        }catch (Exception e){
            XxlJobLogger.log("获取快手创意时报数据失败:服务器内部错误");
            return ReturnT.FAIL;
        }
    }
}
