package org.jeecg;

import cn.com.ctop.common.module.entity.BindAccountLogin;
import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.service.IBindAccountLoginService;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.service.ISendMessageService;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.CtopAdConstant;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouDailyReportTaskService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouHistoryReportTaskService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import cn.com.ctop.kuaishou.modules.graphql.service.IKuaishouWebInterfaceService;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAgentAccount;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAgentAccountMapper;
import cn.com.ctop.kuaishou.modules.report.service.IKuaishouReportDailyAgentService;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceAdvertiserDataService;
import cn.com.ctop.toutiao.modules.report.service.IBytedanceFundDailyService;
import cn.com.ctop.toutiao.modules.report.service.IReportService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.util.DateUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static org.jeecg.common.util.DateUtils.getAnotherDay;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("wps")
@Slf4j
public class SampleTest {
    @Autowired
    private ICtopOauthTokenService oauthTokenService;
    @Autowired
    private IReportService reportService;
    @Autowired
    private IByteDanceAdvertiserDataService advertiserDataService;
    @Autowired
    private IKuaiShouHistoryReportTaskService kuaiShouHistoryReportTaskService;

    @Autowired
    private ICtopOauthTokenService tokenService;

    @Test
    public void loadBytedanceCreativeData() {
        QueryWrapper<CtopOauthToken> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("media_id", 2);
        List<CtopOauthToken> list = oauthTokenService.list(queryWrapper);
        for (CtopOauthToken token : list) {
            kuaiShouHistoryReportTaskService.createTask(token.getAccountId(), token.getAccessToken(), "2020-09-20", "2020-09-20", "daily");
        }
    }

    @Autowired
    private IBindAccountLoginService bindAccountLoginService;
    @Autowired
    private IKuaishouWebInterfaceService kuaishouWebInterfaceService;

    @Test
    public void loadKuaishouCookie() {
        List<BindAccountLogin> list = bindAccountLoginService.getListByParams(CtopAdConstant.PLATFORM_TYPE_KUAISHOU_PY, 1);
        if (list != null && !list.isEmpty()) {
            int i = 0;
            for (BindAccountLogin login : list) {
                if (null == login.getCookie() || "".equals(login.getCookie().trim())) {
                    kuaishouWebInterfaceService.getkuaishouWebLoginCookie(login);
                    i++;
                }
            }
        }
    }

    static ExecutorService executorService = null;
    //线程计数器/bytedance/bytedanceMaterialReport
    static CountDownLatch countDownLatch = null;

    @Test
    public void deleteKuaishouComment() {
        Long start = System.currentTimeMillis();
        List<BindAccountLogin> loginList = bindAccountLoginService.getListByParams(CtopAdConstant.PLATFORM_TYPE_KUAISHOU_PY, 1);
        if (loginList != null && !loginList.isEmpty()) {
            executorService = Executors.newFixedThreadPool(5);
            countDownLatch = new CountDownLatch(loginList.size());
            loginList.forEach(login -> executorService.submit(() -> {
                try {
                    kuaishouWebInterfaceService.deleteAllComment(new HashMap<>(), login);
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                } finally {
                    countDownLatch.countDown();
                }
            }));
            try {
                countDownLatch.await();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            Long end = System.currentTimeMillis();
            log.info("快手删评论所用时长：{}毫秒", end - start);
        }
    }

    @Autowired
    private IUserAllocationService allocationService;

    @Test
    public void testLoadBytedanceData() {
        List<UserAllocation>allocations = allocationService.getByParams(633L,null,0);
        if(null!=allocations&&!allocations.isEmpty()){
            for(UserAllocation allocation:allocations){
                CtopOauthToken token = tokenService.getTokenByAccountId(allocation.getAccountId());
                String startDate = "2020-11-10";
                String endDate = "2020-11-10";
                reportService.getAdvertiserReport(token, DateUtils.parseDate(startDate,"yyyy-MM-dd"), DateUtils.parseDate(endDate,"yyyy-MM-dd"), CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
            }
        }
        System.out.println("任务结束");
    }

    @Test
    public void loadBytedancePlanData(){
        CtopOauthToken token = tokenService.getTokenByAccountId(100198696723L);
//        advertiserDataService.getAdvertiserPlan(token, "", null, null);
        Date getDate = DateUtils.addDay(new Date(), 0);
        reportService.getAdvertiserReport(token, getDate, getDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
    }

    @Autowired
    private IBytedanceFundDailyService bytedanceFundDailyService;
    @Test
    public void loadFoudData() {
        List<CtopOauthToken> tokens = tokenService.selectToutiaoToken();
        for (CtopOauthToken token : tokens) {
            System.out.println(token.getAccountId()+"############################");
            for(int i=20;i<130;i++){
                Date date = DateUtils.addDay(new Date(),-i);
                bytedanceFundDailyService.loadFundDataByPage(token, DateUtils.formatDate(date), DateUtils.formatDate(date),1);
                System.out.println(i+"############################");
            }
        }
    }

    @Autowired
    private IKuaiShouHistoryReportTaskService reportTaskService;

    /**
     * 测试获取快手图片消耗数据任务
     */
    @Test
    public void loadKuaishouImageTask(){
        List<CtopOauthToken> tokens = tokenService.selectKuaiShouToken();
        for(int i=1;i<130;i++) {
            Date getDate = DateUtils.addDay(new Date(), -i);
            String getDateStr = DateUtils.formatDate(getDate);
            tokens.forEach(token -> reportTaskService.createTask(token.getAccountId(), token.getAccessToken(), getDateStr, getDateStr, CtopAdConstant.KUAISHOU_LOAD_JOB_TYPE_DAILY));
        }
    }

    @Autowired
    private IKuaiShouDailyReportTaskService dailyReportTaskService;
    /**
     * 测试获取快手图片消耗数据入库
     */
    @Test
    public void loadKuaishouImageTaskFile(){
        String nowDate = DateUtils.getDate("yyyy-MM-dd");
        List<CtopOauthToken> tokens = tokenService.selectKuaiShouToken();
        for(int i=1;i<15;i++){
            String statDate = getAnotherDay("yyyy-MM-dd", nowDate, -i);
            tokens.forEach(token -> dailyReportTaskService.getTaskList(token.getAccountId(), token.getAccessToken(), statDate));
        }
        //  查询快手token
    }

    @Test
    public void testLoadBytedanceImageData(){
        List<CtopOauthToken>tokens = tokenService.selectToutiaoToken();
        countDownLatch = new CountDownLatch(tokens.size());
        executorService = Executors.newFixedThreadPool(8);
        tokens.forEach(token->executorService.submit(()->{
            try {
                advertiserDataService.getMaterialList(token);
            }catch (Exception e){
            }finally {
                countDownLatch.countDown();
            }
        }));
        try {
            countDownLatch.await();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        System.out.println("素材数据获取完成");

    }
    @Autowired
    private KuaishouAgentAccountMapper kuaishouAgentAccountMapper;
    @Autowired
    private IKuaishouReportDailyAgentService kuaishouReportDailyAgentService;
    @Test
    public void loadKuaishouAgentData() throws Exception{
        kuaishouReportDailyAgentService.loginAgent();
//        for (int i = 0; i < 20; i++) {
//            String currentDate = DateUtils.formatDate(DateUtils.addDay(new Date(), -i));
//            kuaishouReportDailyAgentService.getReport(currentDate, DateUtils.getNowDate("yyyy-MM-dd"));
//        }
//
        try {
            for (int i = 1; i < 30; i++) {
                kuaishouReportDailyAgentService.getAccount(i);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
//        String result1 = "{\"result\":1,\"pageInfo\":{\"currentPage\":1,\"pageSize\":100,\"totalCount\":4509},\"reviewStatusCount\":{\"totalAccount\":4509,\"readToSummit\":14,\"inReview\":9,\"reviewSuccess\":3736,\"reviewFailed\":750},\"adDspAccountList\":[{\"userId\":1134738749,\"accountId\":8213994,\"accountName\":\"MiAo2015\",\"createTime\":1605064051091,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"钻石涂涂乐\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"西安点扣软件科技有限公司\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1134738749,\"accountId\":8213777,\"accountName\":\"MiAo2015\",\"createTime\":1605062758709,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"我眼神儿贼好\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"西安点扣软件科技有限公司\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136789334,\"accountId\":8210669,\"accountName\":\"User_1604999152560\",\"createTime\":1605005597991,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136789334,\"accountId\":8210658,\"accountName\":\"User_1604999152560\",\"createTime\":1605005506195,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136789334,\"accountId\":8210641,\"accountName\":\"User_1604999152560\",\"createTime\":1605005442507,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136789334,\"accountId\":8210630,\"accountName\":\"User_1604999152560\",\"createTime\":1605005346626,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136789334,\"accountId\":8210614,\"accountName\":\"User_1604999152560\",\"createTime\":1605005303898,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210266,\"accountName\":\"啧啧啧1851\",\"createTime\":1605002548471,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210246,\"accountName\":\"啧啧啧1851\",\"createTime\":1605002441860,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210226,\"accountName\":\"啧啧啧1851\",\"createTime\":1605002332665,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210204,\"accountName\":\"啧啧啧1851\",\"createTime\":1605002223512,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210158,\"accountName\":\"啧啧啧1851\",\"createTime\":1605001940380,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210134,\"accountName\":\"啧啧啧1851\",\"createTime\":1605001817125,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210128,\"accountName\":\"啧啧啧1851\",\"createTime\":1605001771128,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210121,\"accountName\":\"啧啧啧1851\",\"createTime\":1605001706321,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210106,\"accountName\":\"啧啧啧1851\",\"createTime\":1605001640969,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":208892593,\"accountId\":8210098,\"accountName\":\"啧啧啧1851\",\"createTime\":1605001578051,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1709082571,\"accountId\":8209735,\"accountName\":\"User_1579866486690\",\"createTime\":1605000295686,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"慢聊\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"盐城爱豆互娱网络科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209269,\"accountName\":\"同城交友\",\"createTime\":1604997820240,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209252,\"accountName\":\"同城交友\",\"createTime\":1604997729500,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209247,\"accountName\":\"同城交友\",\"createTime\":1604997673757,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209239,\"accountName\":\"同城交友\",\"createTime\":1604997599387,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209233,\"accountName\":\"同城交友\",\"createTime\":1604997536986,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209225,\"accountName\":\"同城交友\",\"createTime\":1604997483874,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209208,\"accountName\":\"同城交友\",\"createTime\":1604997391909,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209200,\"accountName\":\"同城交友\",\"createTime\":1604997327774,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209186,\"accountName\":\"同城交友\",\"createTime\":1604997266086,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2052872749,\"accountId\":8209178,\"accountName\":\"同城交友\",\"createTime\":1604997214638,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"陌声\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市微领科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136067779,\"accountId\":8208654,\"accountName\":\"User_1604912636394\",\"createTime\":1604994164574,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136067779,\"accountId\":8208636,\"accountName\":\"User_1604912636394\",\"createTime\":1604994056687,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136067779,\"accountId\":8208613,\"accountName\":\"User_1604912636394\",\"createTime\":1604993959648,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136155765,\"accountId\":8208549,\"accountName\":\"User_1604920009156\",\"createTime\":1604993577946,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"手机淘宝\",\"firstCostDay\":\"-\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136117885,\"accountId\":8208105,\"accountName\":\"User_1604917065523\",\"createTime\":1604990958074,\"balance\":19999886,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":114,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":20000000,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"骏游斗地主\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"棋牌捕鱼\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"杭州骏游科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136117885,\"accountId\":8208079,\"accountName\":\"User_1604917065523\",\"createTime\":1604990868642,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"骏游斗地主\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"棋牌捕鱼\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"杭州骏游科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136117885,\"accountId\":8208049,\"accountName\":\"User_1604917065523\",\"createTime\":1604990767520,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"骏游斗地主\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"棋牌捕鱼\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"杭州骏游科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136117885,\"accountId\":8208040,\"accountName\":\"User_1604917065523\",\"createTime\":1604990719707,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"骏游斗地主\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"棋牌捕鱼\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"杭州骏游科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136117885,\"accountId\":8207989,\"accountName\":\"User_1604917065523\",\"createTime\":1604990563433,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"骏游斗地主\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"棋牌捕鱼\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"杭州骏游科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1597206673,\"accountId\":8207624,\"accountName\":\"甜筒语音APP\",\"createTime\":1604988750908,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"甜筒\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市甜筒网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136566033,\"accountId\":8206788,\"accountName\":\"User_1604973828379\",\"createTime\":1604980780875,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"来来\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"壳壳互联（北京）科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136566033,\"accountId\":8206780,\"accountName\":\"User_1604973828379\",\"createTime\":1604980700594,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"来来\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"壳壳互联（北京）科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136555503,\"accountId\":8206654,\"accountName\":\"小优带你赚钱\uD83C\uDF4A\",\"createTime\":1604979838609,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136555503,\"accountId\":8206639,\"accountName\":\"小优带你赚钱\uD83C\uDF4A\",\"createTime\":1604979724836,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136555503,\"accountId\":8206602,\"accountName\":\"小优带你赚钱\uD83C\uDF4A\",\"createTime\":1604979472189,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136555503,\"accountId\":8206597,\"accountName\":\"小优带你赚钱\uD83C\uDF4A\",\"createTime\":1604979428979,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136555503,\"accountId\":8206587,\"accountName\":\"小优带你赚钱\uD83C\uDF4A\",\"createTime\":1604979373627,\"balance\":49981281,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":285,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":49981566,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136555503,\"accountId\":8206561,\"accountName\":\"小优带你赚钱\uD83C\uDF4A\",\"createTime\":1604979214663,\"balance\":49966652,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":285,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":49966937,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1330563263,\"accountId\":8206438,\"accountName\":\"想你的夜6251\",\"createTime\":1604978417666,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"众安保险\",\"firstCostDay\":\"-\",\"industry\":\"金融服务类\",\"secondIndustry\":\"保险\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"众安在线保险经纪有限公司上海分公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1330563263,\"accountId\":8206424,\"accountName\":\"想你的夜6251\",\"createTime\":1604978319806,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"众安保险\",\"firstCostDay\":\"-\",\"industry\":\"金融服务类\",\"secondIndustry\":\"保险\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"众安在线保险经纪有限公司上海分公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136561913,\"accountId\":8206302,\"accountName\":\"User_1604973369673\",\"createTime\":1604977507582,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"米读极速版\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"影音娱乐\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"上海大犀角信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136562164,\"accountId\":8206288,\"accountName\":\"User_1604973402823\",\"createTime\":1604977430643,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"米读极速版\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"影音娱乐\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"上海大犀角信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136561345,\"accountId\":8206253,\"accountName\":\"User_1604973300364\",\"createTime\":1604977194153,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"米读极速版\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"影音娱乐\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"上海大犀角信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136560487,\"accountId\":8206237,\"accountName\":\"User_1604973191573\",\"createTime\":1604977135375,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"米读极速版\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"影音娱乐\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"上海大犀角信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136151560,\"accountId\":8206061,\"accountName\":\"太空狼人杀大神\",\"createTime\":1604976033138,\"balance\":0,\"creditBalance\":49929110,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":49929110,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"会玩密室杀\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"武汉荟友网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136151560,\"accountId\":8206025,\"accountName\":\"太空狼人杀大神\",\"createTime\":1604975821219,\"balance\":0,\"creditBalance\":49963980,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":49963980,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"会玩密室杀\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"武汉荟友网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131478960,\"accountId\":8203021,\"accountName\":\"黄琪琪\",\"createTime\":1604919579304,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱上来电秀\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136067779,\"accountId\":8202963,\"accountName\":\"User_1604912636394\",\"createTime\":1604919107215,\"balance\":0,\"creditBalance\":49988626,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":49988626,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136067779,\"accountId\":8202900,\"accountName\":\"User_1604912636394\",\"createTime\":1604918584137,\"balance\":0,\"creditBalance\":49971730,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":49971730,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136067779,\"accountId\":8202883,\"accountName\":\"User_1604912636394\",\"createTime\":1604918458987,\"balance\":0,\"creditBalance\":49650387,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":49650387,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2109615889,\"accountId\":8202733,\"accountName\":\"boss直聘2020\",\"createTime\":1604917308086,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"Boss直聘\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"招聘\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"北京华品博睿网络技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136060628,\"accountId\":8202518,\"accountName\":\"User_1604911957779\",\"createTime\":1604915908049,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"计步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136060628,\"accountId\":8202483,\"accountName\":\"User_1604911957779\",\"createTime\":1604915704944,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"计步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136060628,\"accountId\":8202478,\"accountName\":\"User_1604911957779\",\"createTime\":1604915663242,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"计步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136060628,\"accountId\":8202406,\"accountName\":\"User_1604911957779\",\"createTime\":1604915320401,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"计步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2136060628,\"accountId\":8202394,\"accountName\":\"User_1604911957779\",\"createTime\":1604915270646,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"计步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":906455831,\"accountId\":8202341,\"accountName\":\"黑梦。825\",\"createTime\":1604915050572,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊交友\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市爱聊科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":313623265,\"accountId\":8202146,\"accountName\":\"天天健身\",\"createTime\":1604913799407,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"铃声汇\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"南京网眼软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":313623265,\"accountId\":8202115,\"accountName\":\"天天健身\",\"createTime\":1604913614749,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"铃声汇\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"南京网眼软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":313623265,\"accountId\":8202092,\"accountName\":\"天天健身\",\"createTime\":1604913509570,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"铃声汇\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"南京网眼软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":313623265,\"accountId\":8202081,\"accountName\":\"天天健身\",\"createTime\":1604913464508,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"铃声汇\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"南京网眼软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":313623265,\"accountId\":8202072,\"accountName\":\"天天健身\",\"createTime\":1604913410712,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"铃声汇\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"南京网眼软件\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":313623265,\"accountId\":8201887,\"accountName\":\"å¤©å¤©å\u0081¥èº«\",\"createTime\":1604912143252,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"é\u0093\u0083å£°æ±\u0087\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å·¥å\u0085·\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008D\u0097äº¬ç½\u0091ç\u009C¼è½¯ä»¶æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135973659,\"accountId\":8201748,\"accountName\":\"User_1604903185850\",\"createTime\":1604911397394,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"é\u0085\u008Dé\u0085\u008D\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å©\u009Aæ\u0081\u008B/äº¤å\u008F\u008B\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"æ·±å\u009C³é½\u0090äº«å¨±ä¹\u0090ç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135973659,\"accountId\":8201725,\"accountName\":\"User_1604903185850\",\"createTime\":1604911251525,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"é\u0085\u008Dé\u0085\u008D\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å©\u009Aæ\u0081\u008B/äº¤å\u008F\u008B\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"æ·±å\u009C³é½\u0090äº«å¨±ä¹\u0090ç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":1,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2124676103,\"accountId\":8201683,\"accountName\":\"å\u0085\u0085ç\u0094µå¾\u0097å®\u009D\",\"createTime\":1604911036551,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"å\u0085\u0085ç\u0094µå¾\u0097å®\u009D\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å·¥å\u0085·\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å¹¿å·\u009Eå°\u008Fè¿\u0088ç½\u0091ç»\u009Cç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2124676103,\"accountId\":8201673,\"accountName\":\"å\u0085\u0085ç\u0094µå¾\u0097å®\u009D\",\"createTime\":1604910978669,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"å\u0085\u0085ç\u0094µå¾\u0097å®\u009D\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å·¥å\u0085·\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å¹¿å·\u009Eå°\u008Fè¿\u0088ç½\u0091ç»\u009Cç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2124676103,\"accountId\":8201666,\"accountName\":\"å\u0085\u0085ç\u0094µå¾\u0097å®\u009D\",\"createTime\":1604910929384,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"å\u0085\u0085ç\u0094µå¾\u0097å®\u009D\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å·¥å\u0085·\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å¹¿å·\u009Eå°\u008Fè¿\u0088ç½\u0091ç»\u009Cç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2124676103,\"accountId\":8201649,\"accountName\":\"å\u0085\u0085ç\u0094µå¾\u0097å®\u009D\",\"createTime\":1604910861554,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"å\u0085\u0085ç\u0094µå¾\u0097å®\u009D\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å·¥å\u0085·\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å¹¿å·\u009Eå°\u008Fè¿\u0088ç½\u0091ç»\u009Cç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200943,\"accountName\":\"User_1603866467861\",\"createTime\":1604907474745,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200924,\"accountName\":\"User_1603866467861\",\"createTime\":1604907390656,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200895,\"accountName\":\"User_1603866467861\",\"createTime\":1604907291399,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200855,\"accountName\":\"User_1603866467861\",\"createTime\":1604907083650,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200821,\"accountName\":\"User_1603866467861\",\"createTime\":1604906763745,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200795,\"accountName\":\"User_1603866467861\",\"createTime\":1604906616416,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200779,\"accountName\":\"User_1603866467861\",\"createTime\":1604906526468,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200747,\"accountName\":\"User_1603866467861\",\"createTime\":1604906391124,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200697,\"accountName\":\"User_1603866467861\",\"createTime\":1604906148377,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2123800548,\"accountId\":8200666,\"accountName\":\"User_1603866467861\",\"createTime\":1604906010947,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"çº¢æ\u009F\u009Aå\u0085\u008Dè´¹å°\u008Fè¯´\",\"firstCostDay\":\"-\",\"industry\":\"èµ\u0084è®¯ç±»\",\"secondIndustry\":\"å½±é\u009F³å¨±ä¹\u0090\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"å\u008C\u0097äº¬æ\u0097 é\u0099\u0090å\u0085\u0089å\u009Cºç§\u0091æ\u008A\u0080æ\u009C\u0089é\u0099\u0090å\u0085¬å\u008F¸\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1334320441,\"accountId\":8200343,\"accountName\":\"Ajwisjns\",\"createTime\":1604904598259,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"众安保险\",\"firstCostDay\":\"-\",\"industry\":\"金融服务类\",\"secondIndustry\":\"保险\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"众安在线财产保险股份有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1334320441,\"accountId\":8200308,\"accountName\":\"Ajwisjns\",\"createTime\":1604904444465,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"众安保险\",\"firstCostDay\":\"-\",\"industry\":\"金融服务类\",\"secondIndustry\":\"保险\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"众安在线财产保险股份有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8199816,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604901788219,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8199806,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604901704289,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8199773,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604901503947,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8199721,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604901235031,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8199703,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604901092275,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135840557,\"accountId\":8198744,\"accountName\":\"走走逛逛吃吃\",\"createTime\":1604893513629,\"balance\":19982208,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":14951,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":19997159,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"快步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市美君林科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135840557,\"accountId\":8198736,\"accountName\":\"走走逛逛吃吃\",\"createTime\":1604893459390,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"快步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市美君林科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135840557,\"accountId\":8198711,\"accountName\":\"走走逛逛吃吃\",\"createTime\":1604893330812,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"快步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市美君林科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135840557,\"accountId\":8198691,\"accountName\":\"走走逛逛吃吃\",\"createTime\":1604893192169,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"快步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市美君林科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135840557,\"accountId\":8198680,\"accountName\":\"走走逛逛吃吃\",\"createTime\":1604893140250,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"快步宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市美君林科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8197722,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604888804727,\"balance\":18806460,\"creditBalance\":83100222,\"extendedBalance\":0,\"rebate\":114,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":101906796,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]}],\"host-name\":\"ad-rs1833.idczw.hb1.kwaidc.com\"}";
//        String result = "{\"result\":1,\"pageInfo\":{\"currentPage\":2,\"pageSize\":100,\"totalCount\":4509},\"reviewStatusCount\":{\"totalAccount\":4509,\"readToSummit\":14,\"inReview\":9,\"reviewSuccess\":3736,\"reviewFailed\":750},\"adDspAccountList\":[{\"userId\":2135809653,\"accountId\":8197713,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604888762842,\"balance\":46082330,\"creditBalance\":67167293,\"extendedBalance\":0,\"rebate\":285,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":113249908,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8197689,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604888659442,\"balance\":18473758,\"creditBalance\":122216456,\"extendedBalance\":0,\"rebate\":285,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":140690499,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8197682,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604888619235,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2135809653,\"accountId\":8197658,\"accountName\":\"嘟嘟爱喝可乐\",\"createTime\":1604888484223,\"balance\":47442399,\"creditBalance\":67086642,\"extendedBalance\":0,\"rebate\":285,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":114529326,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"优享充电\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1334320441,\"accountId\":8179763,\"accountName\":\"Ajwisjns\",\"createTime\":1604643899271,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"众安保险\",\"firstCostDay\":\"-\",\"industry\":\"金融服务类\",\"secondIndustry\":\"保险\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"众安在线财产保险股份有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132849319,\"accountId\":8179752,\"accountName\":\"User_1604626572731\",\"createTime\":1604643839880,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"众安保险\",\"firstCostDay\":\"-\",\"industry\":\"金融服务类\",\"secondIndustry\":\"保险\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"众安在线保险经纪有限公司上海分公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132849319,\"accountId\":8179743,\"accountName\":\"User_1604626572731\",\"createTime\":1604643779074,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"众安保险\",\"firstCostDay\":\"-\",\"industry\":\"金融服务类\",\"secondIndustry\":\"保险\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"众安在线保险经纪有限公司上海分公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8179394,\"accountName\":\"Liang先生…\",\"createTime\":1604641838243,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8179358,\"accountName\":\"Liang先生…\",\"createTime\":1604641586842,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178842,\"accountName\":\"Liang先生…\",\"createTime\":1604635243151,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178829,\"accountName\":\"Liang先生…\",\"createTime\":1604635124798,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178813,\"accountName\":\"Liang先生…\",\"createTime\":1604635037249,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178805,\"accountName\":\"Liang先生…\",\"createTime\":1604634952719,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178797,\"accountName\":\"Liang先生…\",\"createTime\":1604634879321,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178786,\"accountName\":\"Liang先生…\",\"createTime\":1604634807011,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178765,\"accountName\":\"Liang先生…\",\"createTime\":1604634698413,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178727,\"accountName\":\"Liang先生…\",\"createTime\":1604634527166,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178704,\"accountName\":\"Liang先生…\",\"createTime\":1604634423132,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178681,\"accountName\":\"Liang先生…\",\"createTime\":1604634285020,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178614,\"accountName\":\"Liang先生…\",\"createTime\":1604633794230,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178602,\"accountName\":\"Liang先生…\",\"createTime\":1604633716285,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178544,\"accountName\":\"Liang先生…\",\"createTime\":1604633336122,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178530,\"accountName\":\"Liang先生…\",\"createTime\":1604633285587,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1352111759,\"accountId\":8178498,\"accountName\":\"Liang先生…\",\"createTime\":1604633158531,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"充电得宝\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州米财信息科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132252222,\"accountId\":8177977,\"accountName\":\"走路抽手机\",\"createTime\":1604630051305,\"balance\":0,\"creditBalance\":19903403,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":19903403,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐走\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"鹰隼信息科技（宁波）有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132252222,\"accountId\":8177903,\"accountName\":\"走路抽手机\",\"createTime\":1604629655093,\"balance\":0,\"creditBalance\":19757409,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":19757409,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐走\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"鹰隼信息科技（宁波）有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132252222,\"accountId\":8177893,\"accountName\":\"走路抽手机\",\"createTime\":1604629606943,\"balance\":0,\"creditBalance\":19872403,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":19872403,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐走\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"鹰隼信息科技（宁波）有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132252222,\"accountId\":8177877,\"accountName\":\"走路抽手机\",\"createTime\":1604629542247,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐走\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"鹰隼信息科技（宁波）有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132252222,\"accountId\":8177869,\"accountName\":\"走路抽手机\",\"createTime\":1604629493892,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"欢乐走\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"鹰隼信息科技（宁波）有限公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129502027,\"accountId\":8173909,\"accountName\":\"飞鱼清理官方\",\"createTime\":1604566938608,\"balance\":29971514,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":566,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":29972080,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"飞鱼清理\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"上海极客公社企业发展有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8173200,\"accountName\":\"User_1604304058197\",\"createTime\":1604562715226,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8173187,\"accountName\":\"User_1604304058197\",\"createTime\":1604562660478,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8173141,\"accountName\":\"User_1604304058197\",\"createTime\":1604562326920,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8173127,\"accountName\":\"User_1604304058197\",\"createTime\":1604562229514,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8173115,\"accountName\":\"User_1604304058197\",\"createTime\":1604562160160,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8173046,\"accountName\":\"User_1604304058197\",\"createTime\":1604561750835,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8173040,\"accountName\":\"User_1604304058197\",\"createTime\":1604561704174,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172985,\"accountName\":\"User_1604304058197\",\"createTime\":1604561418705,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172966,\"accountName\":\"User_1604304058197\",\"createTime\":1604561272145,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172951,\"accountName\":\"User_1604304058197\",\"createTime\":1604561186277,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172932,\"accountName\":\"User_1604304058197\",\"createTime\":1604561055204,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172906,\"accountName\":\"User_1604304058197\",\"createTime\":1604560918727,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172863,\"accountName\":\"User_1604304058197\",\"createTime\":1604560614151,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172854,\"accountName\":\"User_1604304058197\",\"createTime\":1604560551725,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172821,\"accountName\":\"User_1604304058197\",\"createTime\":1604560338408,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172775,\"accountName\":\"User_1604304058197\",\"createTime\":1604560028167,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128828507,\"accountId\":8172738,\"accountName\":\"User_1604304058197\",\"createTime\":1604559842023,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"旺旺记账\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132235370,\"accountId\":8172605,\"accountName\":\"太空狼人杀大神\",\"createTime\":1604558846351,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"会玩\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"武汉荟友网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132235370,\"accountId\":8172597,\"accountName\":\"太空狼人杀大神\",\"createTime\":1604558823695,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"会玩\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"武汉荟友网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132235370,\"accountId\":8172550,\"accountName\":\"太空狼人杀大神\",\"createTime\":1604558538800,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"会玩\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"武汉荟友网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132235370,\"accountId\":8172533,\"accountName\":\"太空狼人杀大神\",\"createTime\":1604558463975,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":true,\"productName\":\"会玩\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"武汉荟友网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2132235370,\"accountId\":8172522,\"accountName\":\"太空狼人杀大神\",\"createTime\":1604558411918,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":true,\"productName\":\"会玩\",\"firstCostDay\":\"2020-11-06\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"武汉荟友网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2059566591,\"accountId\":8172118,\"accountName\":\"Cake\",\"createTime\":1604555890982,\"balance\":0,\"creditBalance\":14009554,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":14009554,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"家园保卫战\",\"firstCostDay\":\"2020-11-06\",\"industry\":\"游戏类\",\"secondIndustry\":\"角色扮演\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"天津酷划博达网络技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1285337029,\"accountId\":8171255,\"accountName\":\"张大温柔.Chinese826\",\"createTime\":1604547032007,\"balance\":0,\"creditBalance\":54652400,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":54652400,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"幸福农场\",\"firstCostDay\":\"2020-11-06\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"天津酷划博达网络技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131468369,\"accountId\":8167058,\"accountName\":\"User_1604480565236\",\"createTime\":1604483311995,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"易车\",\"firstCostDay\":\"-\",\"industry\":\"交通类\",\"secondIndustry\":\"汽车资讯与服务平台\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"天津伯优信息技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131468369,\"accountId\":8167020,\"accountName\":\"User_1604480565236\",\"createTime\":1604483152676,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"易车\",\"firstCostDay\":\"-\",\"industry\":\"交通类\",\"secondIndustry\":\"汽车资讯与服务平台\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"天津伯优信息技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131468369,\"accountId\":8167010,\"accountName\":\"User_1604480565236\",\"createTime\":1604483102368,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"易车\",\"firstCostDay\":\"-\",\"industry\":\"交通类\",\"secondIndustry\":\"汽车资讯与服务平台\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"天津伯优信息技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131468369,\"accountId\":8167004,\"accountName\":\"User_1604480565236\",\"createTime\":1604483060825,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"易车\",\"firstCostDay\":\"-\",\"industry\":\"交通类\",\"secondIndustry\":\"汽车资讯与服务平台\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"天津伯优信息技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131468369,\"accountId\":8166978,\"accountName\":\"User_1604480565236\",\"createTime\":1604482973343,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"易车\",\"firstCostDay\":\"-\",\"industry\":\"交通类\",\"secondIndustry\":\"汽车资讯与服务平台\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"天津伯优信息技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1343859197,\"accountId\":8166573,\"accountName\":\"我滴铁汁\",\"createTime\":1604480930275,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"维他奶京东自营旗舰店\",\"firstCostDay\":\"-\",\"industry\":\"电商零售类\",\"secondIndustry\":\"电商商家\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"维他奶（上海）有限公司广州分公司\",\"reviewStatus\":3,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131441858,\"accountId\":8166349,\"accountName\":\"附近交友-未成年慎入\",\"createTime\":1604479797879,\"balance\":0,\"creditBalance\":30330127,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":30330127,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊\",\"firstCostDay\":\"2020-11-06\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市金运网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131441858,\"accountId\":8166287,\"accountName\":\"附近交友-未成年慎入\",\"createTime\":1604479435858,\"balance\":0,\"creditBalance\":31800000,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":31800000,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市金运网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131441858,\"accountId\":8166281,\"accountName\":\"附近交友-未成年慎入\",\"createTime\":1604479386919,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市金运网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131441858,\"accountId\":8166252,\"accountName\":\"附近交友-未成年慎入\",\"createTime\":1604479192190,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市金运网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2131441858,\"accountId\":8166218,\"accountName\":\"附近交友-未成年慎入\",\"createTime\":1604478995743,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市金运网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1241072977,\"accountId\":8165639,\"accountName\":\"User_1549356012591\",\"createTime\":1604475811127,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"汽车之家\",\"firstCostDay\":\"-\",\"industry\":\"交通类\",\"secondIndustry\":\"汽车资讯与服务平台\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"北京车之家信息技术有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164532,\"accountName\":\"User_1604402150137\",\"createTime\":1604470064412,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"-\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164518,\"accountName\":\"User_1604402150137\",\"createTime\":1604470004296,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"-\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164509,\"accountName\":\"User_1604402150137\",\"createTime\":1604469959884,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"-\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":1638871170,\"accountId\":8164502,\"accountName\":\"吃着包子看你浪の\",\"createTime\":1604469921432,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"悟空清理\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"工具\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"上海极客公社企业发展有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164499,\"accountName\":\"User_1604402150137\",\"createTime\":1604469903315,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"-\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164482,\"accountName\":\"User_1604402150137\",\"createTime\":1604469798582,\"balance\":0,\"creditBalance\":29975091,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":29975091,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"-\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164466,\"accountName\":\"User_1604402150137\",\"createTime\":1604469730467,\"balance\":0,\"creditBalance\":29153139,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":29153139,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164448,\"accountName\":\"User_1604402150137\",\"createTime\":1604469659927,\"balance\":0,\"creditBalance\":29082153,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":29082153,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"2020-11-09\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164430,\"accountName\":\"User_1604402150137\",\"createTime\":1604469597582,\"balance\":0,\"creditBalance\":28264196,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":28264196,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"2020-11-07\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164416,\"accountName\":\"User_1604402150137\",\"createTime\":1604469544241,\"balance\":0,\"creditBalance\":131967926,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":131967926,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"2020-11-05\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130800354,\"accountId\":8164402,\"accountName\":\"User_1604402150137\",\"createTime\":1604469449021,\"balance\":0,\"creditBalance\":289161532,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":289161532,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"淘宝特价版\",\"firstCostDay\":\"2020-11-05\",\"industry\":\"电商零售类\",\"secondIndustry\":\"综合电商\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"淘宝(中国)软件有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163799,\"accountName\":\"User_1604369195777\",\"createTime\":1604462940737,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163717,\"accountName\":\"User_1604369195777\",\"createTime\":1604462305459,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163692,\"accountName\":\"User_1604369195777\",\"createTime\":1604462147460,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163657,\"accountName\":\"User_1604369195777\",\"createTime\":1604461992416,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163639,\"accountName\":\"User_1604369195777\",\"createTime\":1604461873974,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163617,\"accountName\":\"User_1604369195777\",\"createTime\":1604461737411,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163609,\"accountName\":\"User_1604369195777\",\"createTime\":1604461695326,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163591,\"accountName\":\"User_1604369195777\",\"createTime\":1604461592502,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163574,\"accountName\":\"User_1604369195777\",\"createTime\":1604461488453,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163558,\"accountName\":\"User_1604369195777\",\"createTime\":1604461393283,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163551,\"accountName\":\"User_1604369195777\",\"createTime\":1604461346467,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163535,\"accountName\":\"User_1604369195777\",\"createTime\":1604461197577,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163525,\"accountName\":\"User_1604369195777\",\"createTime\":1604461140986,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163504,\"accountName\":\"User_1604369195777\",\"createTime\":1604461042813,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163499,\"accountName\":\"User_1604369195777\",\"createTime\":1604460995822,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163479,\"accountName\":\"User_1604369195777\",\"createTime\":1604460911364,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129461653,\"accountId\":8163463,\"accountName\":\"User_1604369195777\",\"createTime\":1604460840930,\"balance\":9978177,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":453,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":9978630,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"猜歌达人\",\"firstCostDay\":\"2020-11-06\",\"industry\":\"游戏类\",\"secondIndustry\":\"休闲益智\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"广州小迈网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130732745,\"accountId\":8159827,\"accountName\":\"同城单身美女速配\",\"createTime\":1604400481716,\"balance\":0,\"creditBalance\":29177696,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":29177696,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊\",\"firstCostDay\":\"2020-11-05\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市金运网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2130732745,\"accountId\":8159820,\"accountName\":\"同城单身美女速配\",\"createTime\":1604400416838,\"balance\":0,\"creditBalance\":31800000,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":31800000,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱聊\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"婚恋/交友\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"深圳市金运网络科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2128767698,\"accountId\":8159532,\"accountName\":\"王者风范＠\",\"createTime\":1604397580986,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"捕鱼大作战\",\"firstCostDay\":\"-\",\"industry\":\"游戏类\",\"secondIndustry\":\"棋牌捕鱼\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"在线途游（北京）科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129490843,\"accountId\":8158956,\"accountName\":\"User_1604372835909\",\"createTime\":1604393785966,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱奇艺随刻\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"影音娱乐\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"北京爱奇艺科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129490843,\"accountId\":8158929,\"accountName\":\"User_1604372835909\",\"createTime\":1604393622425,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱奇艺随刻\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"影音娱乐\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"北京爱奇艺科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]},{\"userId\":2129490843,\"accountId\":8158919,\"accountName\":\"User_1604372835909\",\"createTime\":1604393583879,\"balance\":0,\"creditBalance\":0,\"extendedBalance\":0,\"rebate\":0,\"preRebate\":0,\"contractRebate\":0,\"directRebate\":0,\"totalBalance\":0,\"loLimit\":0,\"singleOut\":0,\"autoOut\":false,\"balanceWarn\":false,\"productName\":\"爱奇艺随刻\",\"firstCostDay\":\"-\",\"industry\":\"资讯类\",\"secondIndustry\":\"影音娱乐\",\"kuaibiBalance\":0,\"pushBalance\":0,\"recharged\":false,\"corporationName\":\"北京爱奇艺科技有限公司\",\"reviewStatus\":2,\"frozenStatus\":0,\"transferAccountStatus\":true,\"reviewDetail\":[]}],\"host-name\":\"ad-rs-sc2288.idcyz.hb1.kwaidc.com\"}";
//        loadData(result1);
//        loadData(result);
    }

    @Autowired
    private ISendMessageService sendMessageService;
    @Test
    public void testSendMessage(){
        sendMessageService.sendMessage("c445fec9a80347aea54e8db66692ec03","测试信息发送！");
    }

    private void loadData(String result)throws Exception{
        ObjectMapper mapper = new ObjectMapper();
        mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        JsonNode resultNode = mapper.readTree(result);
        Integer code = resultNode.get("result").intValue();
        if (code == 1) {
            Iterator<JsonNode> iterator1 = resultNode.get("adDspAccountList").elements();
            int i = 0;
            List<KuaishouAgentAccount> accountList = new ArrayList<>();
            while (iterator1.hasNext()) {
                JsonNode dataNode = iterator1.next();
                KuaishouAgentAccount agentAccount = mapper.readValue(dataNode.toString(), KuaishouAgentAccount.class);
                agentAccount.setBalance(agentAccount.getBalance().divide(new BigDecimal(1000)));
                agentAccount.setContractRebate(agentAccount.getContractRebate().divide(new BigDecimal(1000)));
                agentAccount.setCreditBalance(agentAccount.getCreditBalance().divide(new BigDecimal(1000)));
                agentAccount.setDirectRebate(agentAccount.getDirectRebate().divide(new BigDecimal(1000)));
                agentAccount.setExtendedBalance(agentAccount.getExtendedBalance().divide(new BigDecimal(1000)));
                agentAccount.setKuaibiBalance(agentAccount.getKuaibiBalance().divide(new BigDecimal(1000)));
                agentAccount.setPreRebate(agentAccount.getPreRebate().divide(new BigDecimal(1000)));
                agentAccount.setPushBalance(agentAccount.getPushBalance().divide(new BigDecimal(1000)));
                agentAccount.setRebate(agentAccount.getRebate().divide(new BigDecimal(1000)));
                agentAccount.setTotalBalance(agentAccount.getTotalBalance().divide(new BigDecimal(1000)));
                accountList.add(agentAccount);
            }
            if (!accountList.isEmpty()) {
                kuaishouAgentAccountMapper.replaceBatch(accountList);
            }
        }
    }
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;

    @Test
    public void loadKuaishouGroupData() throws ParseException {
        Date endDate = new Date();
        String anotherDay = DateUtils.getAnotherDay("yyyy-MM-dd", DateUtils.formatDate(endDate), -1);
        SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");
        Date startDate = sim.parse(anotherDay);
        CtopOauthToken token = tokenService.getTokenByAccountId(7087890L);
        kuaishouInterfaceService.getGroupList(token, startDate, endDate);
    }



    @Test
    public void loadAccountData() {
//        List<UserAllocation> allocations = allocationService.getByParams(776L,null,0);
//        for (UserAllocation allocation:allocations) {
            CtopOauthToken token = tokenService.getTokenByAccountId(93238965516L);
            for(int i=92;i<100;i++){
                Date getDate = DateUtils.addDay(new Date(),-i);
                reportService.getAdvertiserReport(token, getDate, getDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
            }
//        }
    }
}
